/* Copyright (C) 2007-2022 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Sergey Gorbunov [committer], Igor Kulakov, Sergei Zharko */

#ifndef L1Station_h
#define L1Station_h 1

#include <string>
#include <type_traits>

#include "L1Field.h"
#include "L1Material.h"
#include "L1SimdSerializer.h"
#include "L1UMeasurementInfo.h"
#include "L1Undef.h"
#include "L1Utils.h"
#include "L1XYMeasurementInfo.h"

/// Structure L1Station
/// Contains a set of geometry parameters for a particular station
///
class L1Station {
public:
  // TODO: SZh 12.05.2022: Rewrite type into L1DetectorID, change detector indexing scheme
  // TODO: SZh 12.05.2022: Provide getters to stations

  int type        = undef::kI32;
  int timeInfo    = undef::kI32;  ///< flag: if time information can be used
  int fieldStatus = undef::kI32;  ///< flag: 1 - station is INSIDE the field, 0 - station is OUTSIDE the field
  fvec fZ         = undef::kFvc;  ///< z position of station     [cm]
  fvec Xmax       = undef::kFvc;  ///< min radius of the station [cm]
  fvec Ymax       = undef::kFvc;  ///< max radius of the station [cm]

  L1FieldSlice fieldSlice {};

  /// Serialization function
  friend class boost::serialization::access;
  template<class Archive>
  void serialize(Archive& ar, const unsigned int)
  {
    ar& type;
    ar& timeInfo;
    ar& fieldStatus;

    ar& fZ;
    ar& Xmax;
    ar& Ymax;

    ar& fieldSlice;
  }


  /// Prints object fields
  /// \param verbosity  Verbosity level of the output
  void Print(int verbosity = 0) const;

  /// String representation of class contents
  /// \param verbosityLevel  Verbosity level of the output
  /// \param indentLevel     Number of indent characters in the output
  std::string ToString(int verbosityLevel = 0, int indentLevel = 0) const;

  /// Verifies class invariant consistency
  /// \note Object is considered undefined in the creation time, so this function should be called after the object
  /// initialization
  void CheckConsistency() const;

} _fvecalignment;


#endif  // L1Station_h
